#include <Windows.h>
#include <atlbase.h>
#include <streams.h>
//#include <dvdmedia.h>
#include "MIDL\IMemoryRenderer.h"
#include "MemoryRendererAllocator.h"
#include "MemoryRendererInputPin.h"
#include "MemoryRenderer.h"


// RXgN^
MemoryRenderer::MemoryRenderer( LPCTSTR tszName, LPUNKNOWN pUnknown, HRESULT *phr ) :
	CBaseVideoRenderer( CLSID_MemoryRenderer, tszName, pUnknown, phr )
	,inputPin( this, &m_InterfaceLock, phr, L"Input" )
{
	this->pBuffer = NULL;
	this->nBufferSize = 0;
	this->nWidth = 0;
	this->nHeight = 0;
	ZeroMemory( &this->guidCennected, sizeof(this->guidCennected) );
	CBaseRenderer::m_pInputPin = &(this->inputPin);
	this->bFoundMediaType = false;
}

// fXgN^
MemoryRenderer::~MemoryRenderer()
{
	// obt@̉B
	if( this->pBuffer != NULL )
		free( this->pBuffer );

	//ꁫȂ CBaseRenderer ̃fXgN^deleteĂ܂̂ŒӁI
    CBaseRenderer::m_pInputPin = NULL;
}



// w肵fBA^Cv󂯕t邩ǂ̔f
HRESULT MemoryRenderer::CheckMediaType( const CMediaType *pMediaType )
{
	if( this->bFoundMediaType ||
		*pMediaType->Type() != MEDIATYPE_Video )
		return S_FALSE;

	if( ( pMediaType->formattype == FORMAT_VideoInfo ) &&
		( pMediaType->cbFormat >= sizeof( VIDEOINFOHEADER ) ) &&
		( pMediaType->pbFormat != NULL ) )
	{
		LPBITMAPINFOHEADER pBMPHeader = &(((VIDEOINFOHEADER*)pMediaType->pbFormat)->bmiHeader);

		// XRGB32
		if(	( pMediaType->subtype == MEDIASUBTYPE_RGB32 ) &&
			( pBMPHeader->biCompression == BI_RGB ) &&
			( pBMPHeader->biBitCount == 32 ) )
		{
			this->guidCennected = pMediaType->subtype;
			this->bFoundMediaType = true;
			return S_OK;
		}
		/*
		// NV12
		if(	( pMediaType->subtype == MEDIASUBTYPE_NV12 ) &&
			( pBMPHeader->biBitCount == 12 ) )
		{
			this->guidCennected = pMediaType->subtype;
			this->bFoundMediaType = true;
			return S_OK;
		}
		*/
	}
	/*
	else if(
		( pMediaType->formattype == FORMAT_VideoInfo2 ) &&
		( pMediaType->cbFormat >= sizeof( VIDEOINFOHEADER2 ) ) &&
		( pMediaType->pbFormat != NULL ) )
	{
		LPBITMAPINFOHEADER pBMPHeader = &(((VIDEOINFOHEADER2*)pMediaType->pbFormat)->bmiHeader);

	}
	*/

	// ͔̑Ή
	return S_FALSE;
}

// s̃fBA^Cv̐ݒ
HRESULT MemoryRenderer::SetMediaType( const CMediaType *pMediaType )
{
	LPBITMAPINFOHEADER pBMPHeader = NULL;
	if( ( pMediaType->formattype == FORMAT_VideoInfo ) &&
		( pMediaType->cbFormat >= sizeof( VIDEOINFOHEADER ) ) &&
		( pMediaType->pbFormat != NULL ) )
	{
		pBMPHeader = &(((VIDEOINFOHEADER*)pMediaType->pbFormat)->bmiHeader);
	}
	else
	{
		pBMPHeader = NULL;
		//pBMPHeader = &(((VIDEOINFOHEADER2*)pMediaType->pbFormat)->bmiHeader);
	}

	// TCY̎擾B
	this->nWidth = pBMPHeader->biWidth;
	this->nHeight = pBMPHeader->biHeight;
	if( this->nHeight < 0 )
		this->nHeight = -this->nHeight;
	
	// obt@̊mہB
	this->nBufferSize = this->nWidth * this->nHeight * pBMPHeader->biBitCount / 8;
	this->pBuffer = (void*) malloc( this->nBufferSize );	

	return CBaseVideoRenderer::SetMediaType( pMediaType );
}

// Tṽ_O
HRESULT MemoryRenderer::DoRenderSample( IMediaSample *pMediaSample )
{
	CAutoLock cSampleLock( &latestSampleLock );
	
	if( this->inputPin.ActiveGrabAllocator() )
	{
		// (A) ÕAP[^gĂꍇ́ApMediaSample ̖{̂iCGrabRendererAllocatorjĂ̂ŁARs[|C^ۑĂB
		//     ̂ƂAm_buffer ͍XVȂB
		
		this->pLatestSample = pMediaSample;
	}
	else
	{
		// (B) ÕAP[^gĂȂꍇ́ApMediaSample ̓eׂ m_buffer ɃRs[ĂB
		//     ̂ƂAm_pLatestSample ͍XVȂB
		
		HRESULT hr = this->Grab( pMediaSample, this->nBufferSize, this->pBuffer );
		if( FAILED( hr ) )
			return hr;
	}

	return S_OK;
}

// ŐṼTv̊l
HRESULT MemoryRenderer::Grab( IMediaSample* pSample, long nBufferSize, void* pBuffer )
{
	// ̃\bhł CAutoLock ȂƁifbhbNɊׂjB
	// ̑Ã\bȟĂяo CAutoLock 邱ƁB

	if( pSample && pBuffer )
	{
		const long nSampleSize = pSample->GetSize();
		
		if( nSampleSize > nBufferSize )
			return E_OUTOFMEMORY;

		BYTE *pSrcBuffer;
		pSample->GetPointer( &pSrcBuffer );

		CopyMemory( pBuffer, pSrcBuffer, nSampleSize );		// Tv́AʂɃt@CRs[iϊjB
	}

	return S_OK;
}


//  override  IMemoryRenderer ɃLXgł悤ɂB
STDMETHODIMP MemoryRenderer::NonDelegatingQueryInterface( REFIID riid, void **ppv )
{
	if( riid == IID_IMemoryRenderer )
		return GetInterface( (IMemoryRenderer*) this, ppv );

	return CBaseVideoRenderer::NonDelegatingQueryInterface( riid, ppv );
}


// ݂̃obt@f[^]
STDMETHODIMP MemoryRenderer::GetCurrentBuffer( byte* pBuffer, long nBufferSize )
{
	CAutoLock cSampleLock( &latestSampleLock );

	if( this->inputPin.ActiveGrabAllocator() )
	{
		// (A) ÕAP[^gĂꍇ́Am_pLatestSample w肳ꂽ pBuffer ɃRs[B
	
		HRESULT hr = this->Grab( this->pLatestSample, nBufferSize, pBuffer );
		if( FAILED( hr ) )
			return hr;
	}
	else
	{
		// (B) ÕAP[^gĂȂꍇ́Am_buffer w肳ꂽ pBuffer ɃRs[B
		
		if( nBufferSize < this->nBufferSize )
			return E_OUTOFMEMORY;

		CopyMemory( pBuffer, this->pBuffer, this->nBufferSize );
	}

	return S_OK;
}
